import numpy as np
from ntsim.IO.gPhotons import gPhotons
from ntsim.PrimaryGenerators.Base.PrimaryGeneratorBase import PrimaryGeneratorBase
from ntsim.random import rng

class IsotropicSource(PrimaryGeneratorBase):
    arg_dict={'position_m': {'type': float , 'nargs': 3 , 'default': [0.,0.,0.] , 'help': 'position of point-like source'},
              'n_photons': {'type': float ,  'default': 100000 , 'help': 'amount of photons to generate'},
              'n_bunches': {'type': int ,  'default': 1 , 'help': 'amount of photons'},
              'photons_weight': {'type': int , 'default': 10000 , 'help': 'amount of bunches'},
              'wavelength_nm': {'type': float , 'default': 400 , 'help': 'photons wavelength'}}
    
    def make_event(self, event):
        event.photons = self.make_photons_generator()

    def make_direction(self):
        dir = np.empty(shape=(1, self.photons_in_bunch, 3))
        phi = rng.uniform(0 , 2*np.pi ,self.photons_in_bunch)
        theta = rng.uniform(0 , np.pi , self.photons_in_bunch)
        dir[:,:,0] = np.cos(phi) * np.sin(theta)
        dir[:,:,1] = np.sin(phi) * np.sin(theta)
        dir[:,:,2] = np.cos(theta)
        return dir

    def make_photons(self):
        dir = self.make_direction()
        position = np.tile(self.position_m, (1,self.photons_in_bunch,1))
        t = np.zeros((1,self.photons_in_bunch))
        wv = np.tile(self.wavelength_nm, self.photons_in_bunch)
        photons = gPhotons()
        photons.add_photons(new_n_photons=self.photons_in_bunch, new_n_steps=1, new_position_m=position, new_t_ns=t, 
                            new_direction=dir, new_wavelength_nm=wv, new_weight=self.photons_weight, new_progenitor=[0])
        return photons

    def make_photons_generator(self):
        self.photons_in_bunch = int(self.n_photons / self.n_bunches)
        for i in range(self.n_bunches):
            yield self.make_photons()
            
